/* vi:set sw=4

    xclipstr.c

    $Id: xclipstr.c,v 1.2 2002/06/18 17:10:41 aamine Exp $

*/

#include <stdio.h>
#include <stdlib.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#include <X11/Xlocale.h>

static Display *display;
static Window window;
static int selected = 0;
static Atom compound_text;

static void initialize(void);
static void finalize(void);
static void terminate(int status);
static void own_selection(XEvent event);
static void store_selection_data(XEvent event, char *str);
static void d(char *str);

int
main(int argc, char **argv)
{
    XEvent event;
    char *copydata;

    if (argc < 2) {
        fputs("argument not given\n", stderr);
        fputs("usage: xclipstr <string>\n", stderr);
        exit(1);
    }
    copydata = argv[1];

    initialize();

    event.xbutton.time = 0;
    own_selection(event);
    d("own");
    while (1) {
        d("try XNextEvent");
        XNextEvent(display, &event);
        d("got event");
        switch (event.type) {
        case SelectionClear:
            d("EVENT: selection clear");
            if (selected) {
                terminate(0);
            }
            break;
        case SelectionRequest:
            d("EVENT: selection request");
            if (selected) {
                d("we are selection owner");
                store_selection_data(event, copydata);
            }
            else {
                d("not selecting");
            }
            break;
        default:
            d("EVENT: other");
            break;
        }
    }
}

static void
initialize()
{
    setlocale(LC_ALL, "");

    display = XOpenDisplay(NULL);
    /* A window is needed to own selection. */
    window = XCreateSimpleWindow(display, RootWindow(display, 0),
                                 10, 10, 10, 10,
                                 2, BlackPixel(display, 0),
                                 WhitePixel(display, 0));
    compound_text = XInternAtom(display, "COMPOUND_TEXT", False);
}

static void
finalize(void)
{
    XDestroyWindow(display, window);
    XCloseDisplay(display);
}

static void
terminate(int status)
{
    finalize();
    exit(status);
}

static void
own_selection(XEvent event)
{
    XSetSelectionOwner(display, XA_PRIMARY, window, event.xbutton.time);
    if (! XGetSelectionOwner(display, XA_PRIMARY) == window) {
        /* What can I do? */
        d("could not get selection");
        terminate(3);
    }
    selected = 1;
}

static void
store_selection_data(XEvent event, char *str)
{
    XSelectionEvent e;

    e.type = SelectionNotify;
    e.requestor = event.xselectionrequest.requestor;
    e.selection = event.xselectionrequest.selection;
    e.target    = event.xselectionrequest.target;
    e.time      = event.xselectionrequest.time;
    e.property  = event.xselectionrequest.property;

    if (event.xselectionrequest.selection == XA_PRIMARY &&
        event.xselectionrequest.target == XA_STRING) {

        d("selection matched (XA_PRIMARY / XA_STRING)");
        XChangeProperty(display, e.requestor, e.property, e.target,
                        8, PropModeReplace, str, strlen(str));
    }
    else if (event.xselectionrequest.selection == XA_PRIMARY &&
             event.xselectionrequest.target == compound_text) {
        char buf[1024];
        char *list[1];
        XTextProperty ctext;

        d("selection matched (XA_PRIMARY / compound text)");
        strcpy(buf, str);
        list[0] = buf;
        XmbTextListToTextProperty(display, list, 1, XCompoundTextStyle, &ctext);
        XChangeProperty(display, e.requestor, e.property, e.target,
                        8, PropModeReplace, ctext.value, ctext.nitems);
    }
    else {
        if (event.xselectionrequest.selection == XA_PRIMARY)
            d("!!! selection NOT matched (is XA_PRIMARY) !!!");
        else
            d("!!! selection NOT matched (not XA_PRIMARY) !!!");
        e.property = None;
    }
    XSendEvent(display, e.requestor, False, 0, (XEvent*)&e);
}

static void
d(char *str)
{
#ifdef DEBUG
    fprintf(stderr, "%s\n", str);
#endif
}
